Module: Mongoid::Changeable
Overview
Defines behavior for dirty tracking.
Defined Under Namespace
Modules: ClassMethods
Instance Method Summary collapse
-
#attribute_before_last_save(attr) ⇒ Object
Returns the original value of an attribute before the last save.
-
#changed ⇒ Array<String>
Get the changed attributes for the document.
-
#changed? ⇒ true | false
Has the document changed?.
-
#changed_attributes ⇒ Hash<String, Object>
Get the attribute changes.
-
#changes ⇒ Hash<String, Array<Object, Object> ] The changes.
Get all the changes for the document.
-
#children_changed? ⇒ true | false
Have any children (embedded documents) of this document changed?.
-
#move_changes ⇒ Object
Call this method after save, so the changes can be properly switched.
-
#post_persist ⇒ Object
Things that need to execute after a document has been persisted.
-
#previous_changes ⇒ Hash<String, Array<Object, Object> ] The previous changes.
Get the previous changes on the document.
-
#remove_change(name) ⇒ Object
Remove a change from the dirty attributes hash.
-
#saved_change_to_attribute(attr) ⇒ Array<Object> | nil
Returns the change to an attribute during the last save.
-
#saved_change_to_attribute?(attr, **kwargs) ⇒ true | false
Returns whether this attribute changed during the last save.
-
#setters ⇒ Hash
Gets all the new values for each of the changed fields, to be passed to a MongoDB $set modifier.
-
#will_save_change_to_attribute?(attr, **kwargs) ⇒ true | false
Returns whether this attribute change the next time we save.
Instance Method Details
#attribute_before_last_save(attr) ⇒ Object
Returns the original value of an attribute before the last save.
This method is useful in after callbacks to get the original value of
an attribute before the save that triggered the callbacks to run.
146 147 148 149 |
# File 'lib/mongoid/changeable.rb', line 146 def attribute_before_last_save(attr) attr = database_field_name(attr) attributes_before_last_save[attr] end |
#changed ⇒ Array<String>
Get the changed attributes for the document.
15 16 17 |
# File 'lib/mongoid/changeable.rb', line 15 def changed changed_attributes.keys.select { |attr| attribute_change(attr) } end |
#changed? ⇒ true | false
Has the document changed?
25 26 27 |
# File 'lib/mongoid/changeable.rb', line 25 def changed? changes.values.any? { |val| val } || children_changed? end |
#changed_attributes ⇒ Hash<String, Object>
Get the attribute changes.
44 45 46 |
# File 'lib/mongoid/changeable.rb', line 44 def changed_attributes @changed_attributes ||= {} end |
#changes ⇒ Hash<String, Array<Object, Object> ] The changes.
Get all the changes for the document.
54 55 56 57 58 59 60 61 |
# File 'lib/mongoid/changeable.rb', line 54 def changes _changes = {} changed.each do |attr| change = attribute_change(attr) _changes[attr] = change if change end _changes.with_indifferent_access end |
#children_changed? ⇒ true | false
This intentionally only considers children and not descendants.
Have any children (embedded documents) of this document changed?
34 35 36 |
# File 'lib/mongoid/changeable.rb', line 34 def children_changed? _children.any?(&:changed?) end |
#move_changes ⇒ Object
Call this method after save, so the changes can be properly switched.
This will unset the memoized children array, set new record flag to false, set the document as validated, and move the dirty changes.
70 71 72 73 74 75 76 77 78 79 |
# File 'lib/mongoid/changeable.rb', line 70 def move_changes @changes_before_last_save = @previous_changes @previous_changes = changes @attributes_before_last_save = @previous_attributes @previous_attributes = attributes.dup Atomic::UPDATES.each do |update| send(update).clear end changed_attributes.clear end |
#post_persist ⇒ Object
Things that need to execute after a document has been persisted.
85 86 87 88 89 |
# File 'lib/mongoid/changeable.rb', line 85 def post_persist reset_persisted_descendants reset_attributes_before_type_cast move_changes end |
#previous_changes ⇒ Hash<String, Array<Object, Object> ] The previous changes.
Get the previous changes on the document.
97 98 99 |
# File 'lib/mongoid/changeable.rb', line 97 def previous_changes @previous_changes ||= {} end |
#remove_change(name) ⇒ Object
Remove a change from the dirty attributes hash. Used by the single field atomic updaters.
108 109 110 |
# File 'lib/mongoid/changeable.rb', line 108 def remove_change(name) changed_attributes.delete(name.to_s) end |
#saved_change_to_attribute(attr) ⇒ Array<Object> | nil
Returns the change to an attribute during the last save.
157 158 159 160 |
# File 'lib/mongoid/changeable.rb', line 157 def saved_change_to_attribute(attr) attr = database_field_name(attr) previous_changes[attr] end |
#saved_change_to_attribute?(attr, **kwargs) ⇒ true | false
Returns whether this attribute changed during the last save.
This method is useful in after callbacks, to see the change
in an attribute during the save that triggered the callbacks to run.
174 175 176 177 178 179 180 181 182 183 184 185 186 |
# File 'lib/mongoid/changeable.rb', line 174 def saved_change_to_attribute?(attr, **kwargs) changes = saved_change_to_attribute(attr) return false unless changes.is_a?(Array) if kwargs.key?(:from) && kwargs.key?(:to) changes.first == kwargs[:from] && changes.last == kwargs[:to] elsif kwargs.key?(:from) changes.first == kwargs[:from] elsif kwargs.key?(:to) changes.last == kwargs[:to] else true end end |
#setters ⇒ Hash
Gets all the new values for each of the changed fields, to be passed to a MongoDB $set modifier.
121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 |
# File 'lib/mongoid/changeable.rb', line 121 def setters mods = {} changes.each_pair do |name, changes| if changes old, new = changes field = fields[name] key = atomic_attribute_name(name) if field && field.resizable? field.add_atomic_changes(self, name, key, mods, new, old) else mods[key] = new unless atomic_unsets.include?(key) end end end mods end |
#will_save_change_to_attribute?(attr, **kwargs) ⇒ true | false
Returns whether this attribute change the next time we save.
This method is useful in validations and before callbacks to determine
if the next call to save will change a particular attribute.
200 201 202 |
# File 'lib/mongoid/changeable.rb', line 200 def will_save_change_to_attribute?(attr, **kwargs) attribute_changed?(attr, **kwargs) end |