Module: Mongoid::Encryptable::ClassMethods

Defined in:
lib/mongoid/encryptable.rb

Instance Method Summary collapse

Instance Method Details

#encrypt_with(options = {}) ⇒ Object

Set the encryption metadata for the model. Parameters set here will be used to encrypt the fields of the model, unless overridden on the field itself.

is deterministic or not.

Parameters:

  • options (Hash) (defaults to: {})

    The encryption metadata.

Options Hash (options):

  • :key_id (String)

    The base64-encoded UUID of the key used to encrypt fields.

  • :deterministic (true | false)

    Whether the encryption



23
24
25
# File 'lib/mongoid/encryptable.rb', line 23

def encrypt_with(options = {})
  self. = options
end

#encrypted?true | false

Whether the model is encrypted. It means that either the encrypt_with method was called on the model, or at least one of the fields is encrypted.

Returns:

  • (true | false)

    Whether the model is encrypted.



32
33
34
# File 'lib/mongoid/encryptable.rb', line 32

def encrypted?
  !.empty? || fields.any? { |_, field| field.is_a?(Mongoid::Fields::Encrypted) }
end

#set_key_id(key_id) ⇒ Object

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

Override the key_id for the model.

This method is solely for testing purposes and should not be used in the application code. The schema_map is generated very early in the application lifecycle, and overriding the key_id after that will not have any effect.



44
45
46
# File 'lib/mongoid/encryptable.rb', line 44

def set_key_id(key_id)
  self.[:key_id] = key_id
end