Class: Mongoid::RawValue
- Inherits:
-
Object
- Object
- Mongoid::RawValue
- Defined in:
- lib/mongoid/extensions/raw_value.rb
Instance Attribute Summary collapse
-
#raw_value ⇒ Object
readonly
Returns the value of attribute raw_value.
Instance Method Summary collapse
-
#initialize(raw_value) ⇒ RawValue
constructor
A new instance of RawValue.
-
#inspect ⇒ String
Returns a string containing a human-readable representation of the object, including the inspection of the underlying value.
Constructor Details
#initialize(raw_value) ⇒ RawValue
Returns a new instance of RawValue.
20 21 22 |
# File 'lib/mongoid/extensions/raw_value.rb', line 20 def initialize(raw_value) @raw_value = raw_value end |
Instance Attribute Details
#raw_value ⇒ Object (readonly)
Returns the value of attribute raw_value.
18 19 20 |
# File 'lib/mongoid/extensions/raw_value.rb', line 18 def raw_value @raw_value end |
Instance Method Details
#inspect ⇒ String
Returns a string containing a human-readable representation of the object, including the inspection of the underlying value.
28 29 30 |
# File 'lib/mongoid/extensions/raw_value.rb', line 28 def inspect "RawValue: #{raw_value.inspect}" end |